package basis;

import logic.RndMCP;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.BitSet;

import util.FileOpen;
import util.UtiltyFunction;
import util.score.ExposureCounter;

public abstract class BaseTestHybridRBPGenerator extends BaseTestGenerator {
    protected int _C1;
    protected double _C2;
    protected double _timeOver;
    protected ArrayList<BitSet> CurrentClique;
    public boolean AlreadyInitSearch;
    public int LpTotalVertices = 0;
    public int LpTotalSolutions = 0;

    private final String _tempFolderName = ".//TempData//";
    private final String _tempFileBaseName = "TempSave.csv";

    private String PATH = ".//";
    private String LOG_PATH = PATH + "log//";

    public int ParallelNum = 1;
    public int subGraphSize = 1;
    public boolean ipLbInit = false;

    public void SetRndMCPConstrain(int c1, double c2, double timeOver) {
        this._C1 = c1;
        this._C2 = c2;
        this._timeOver = timeOver;
    }

    protected void InitialSearch() {
        if (this.AlreadyInitSearch) {
            return;
        }

        RndMCP rnd = new RndMCP(this._C1, this._C2, this._timeOver);
        rnd.SetConstraint(super.constraint);
        rnd.Generate();

        if (rnd.result != null && 0 < rnd.result.size()) {
            this.CurrentClique = rnd.result;
            super.result = (ArrayList<BitSet>) this.CurrentClique.clone();
            this.OutputTempTests();
            System.out.println("InitializeResult" + rnd.result.size());
            System.out.println("Random");
        } else {
            this.CurrentClique = new ArrayList<>();
            super.result = (ArrayList<BitSet>) this.CurrentClique.clone();
        }
    }

    public void Generate() {
        super.Generate();
    }

    public void PrePreprocess() {
        this.AlreadyInitSearch = true;
        if (this.AlreadyInitSearch) {
            this.ReadTests("");
            this.ExportCsv(0, 0, 0, System.currentTimeMillis() - ((long)this._timeOver));
        } else {
            this.ExportCsv(0, 0 , 0, System.currentTimeMillis());
        }
        if (!super.InitializeCheck()) {
            return;
        }
    }

    public void ReadTests(String high) {

        String fileName = ".//TempData//" + "i" + high + "_" + this.constraint.getItemBank().size() + "_" + this.constraint.NumOfItems.get(0) + "_" + this.constraint.UBofOverlappingItems.get(0) + "" + ".csv";

        String[][] data = FileOpen.loadDataString(fileName);

        ArrayList<BitSet> tests = new ArrayList<>();

        for (int j = 0; j < data.length; j++) {
            int k = 0;
            BitSet test = new BitSet();
            for (int i = 0; i < data[j].length; i++) {
                do {
                    if (data[j][i].equals(this.constraint.getItemBank().get(k).id)) {
                        test.set(k);
                        k++;
                        break;
                    } else {
                        test.clear(k);
                        k++;
                    }
                } while(true);
            }
            tests.add(test);


            if (j % 1000 == 0) {
                System.out.println();
            } else if (j % 100 == 0) {
                System.out.print("|");
            }
        }

        this.CurrentClique = (ArrayList<BitSet>) tests.clone();
        this.result = (ArrayList<BitSet>) tests.clone();
    }

    public void OutputTempTests() {

        String FileFullPath = _tempFolderName + super.constraint.getItemBank().size() + "_" +
                super.constraint.NumOfItems.get(0) + "_" + super.constraint.UBofOverlappingItems.get(0) + _tempFileBaseName;
        PrintWriter pt = null;
        try {
            pt = new PrintWriter(FileFullPath);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        for (BitSet test : this.CurrentClique) {
            for (int i = 0; i < test.size(); i++) {
                if (i != 0) {
                    pt.print(",");
                }
                pt.print(Boolean.TRUE.compareTo(test.get(i)));
            }
            pt.println();
        }
        pt.close();
    }

    public void ExportCsv(int solveSize, int cliqueSize, int currentSize, long timeMillis) {

        try {
            String fileName = LOG_PATH + super.method.name() + "_Bank" + super.constraint.getItemBank().size() + "_Len" +
                    super.constraint.NumOfItems.get(0) + "_OC" + super.constraint.UBofOverlappingItems.get(0) + "_" +
                    "_SubSize" + String.valueOf(this.subGraphSize) + "_LB" + String.valueOf(this.ipLbInit) +
                    "ParallelNumber" + "_" + String.valueOf(this.ParallelNum) + "_" + "SearchLog.csv";

            FileWriter f = new FileWriter(fileName, true);
            PrintWriter p = new PrintWriter(new BufferedWriter(f));

            p.print(solveSize);
            p.print(",");
            p.print(cliqueSize);
            p.print(",");
            p.print(currentSize + cliqueSize);
            p.print(",");
            if (super.result != null) {
                if (currentSize + cliqueSize < super.result.size()) {
                    p.print(super.result.size());
                } else {
                    p.print(currentSize + cliqueSize);
                }
            } else {
                p.print(0);
            }
            p.print(",");
            p.print(timeMillis);
            p.println();

            p.close();

        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }

    public void ExportLstCsv(int pn) {

        try {
            String fileName = LOG_PATH + super.method.name() + "_Bank" + super.constraint.getItemBank().size() +
                    "_Len" + super.constraint.NumOfItems.get(0) + "_OC" + super.constraint.UBofOverlappingItems.get(0) +
                    "_SubSize" + String.valueOf(this.subGraphSize) + "_LB" + String.valueOf(this.ipLbInit) +
                    "_Parallel" + String.valueOf(this.ParallelNum) + "SearchLogFinal.csv";

            FileWriter f = new FileWriter(fileName, true);
            PrintWriter p = new PrintWriter(new BufferedWriter(f));

            p.print(pn);
            p.print(",");
            p.print(this.LpTotalSolutions);
            p.print(",");
            p.print(this.LpTotalVertices);
            p.print(",");
            p.print(System.currentTimeMillis());
            p.println();

            p.close();

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void OutputResult(String FileName) {

        String FileFullPath = this.OUTPUT_FILE_PATH + UtiltyFunction.GetFileName() + method + "_" + FileName;
        PrintWriter pt = null;
        try {
            pt = new PrintWriter(FileFullPath);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        /*
        for (int i = 0; i < this.constraint.Fields.size(); i++) {
            pt.println(this.constraint.Fields.get(i) + "," + this.constraint.NumOfItems.get(i) + "," +
                            this.constraint.UBofOverlappingItems.get(i) + "," + String.valueOf(this.subGraphSize) + "," +
                            String.valueOf(this.ipLbInit) + "," + String.valueOf(this.ParallelNum));
        }
        pt.println("theta,LB of I(theta), UB of I(theta)");
        for (int iii = 0; iii < this.constraint.theta.size(); iii++) {
            pt.print(this.constraint.theta.get(iii) + ",");
            pt.print(this.constraint.InformationUB.get(iii) + ",");
            pt.println(this.constraint.InformationLB.get(iii));
        }


        ExposureCounter e = new ExposureCounter();
        e.SetData(this.constraint.getItemBank());
        pt.println("構成テスト数," + this.result.size());
        pt.println("露出度," + e.Score(this.result));

        pt.println("以下構成テスト");
        pt.println("-----");
         */

        IDGen();
        for (int i = 0; i < this.result.size(); i++) {
            pt.println(this.resultIds.get(i));
        }
        //pt.println("-----");

        pt.close();

    }
}
