package basis;

import logic.*;
import util.*;
import basis.EnumerationType.MethodName;

import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.ArrayList;

public class BaseTester extends Thread implements ITester {
    public static class TestConstrain {
        public String Size;
        public String ConstrainCsv;
        public EnumerationType.MethodName _MethodName;
        public double TimeOver;

        public int ParallelNumber;
        public int ThreadNumber;
        public EnumerationType.ParallelSearchType SearchType;

        public int subGraphSize;
        public boolean ipInitLb;

        public int L1;
        public double L2;
        public double CT_prime;
        public boolean hasStep1Solutions;
    }

    public static class TestConstrains {
        public static ArrayList<TestConstrain> _TestConstrains;
        
        static {
            _TestConstrains = new ArrayList<>();
            TestConstrain constrain;

            double timeOver_1 = 1000.0 * 60;
            String const_name = "IEEE_25item_0overlap.csv";

            constrain = new TestConstrain();
            constrain.Size = "1000";
            constrain.ConstrainCsv = const_name;
            constrain._MethodName = MethodName.RndMCP;
            constrain.TimeOver = timeOver_1;
            constrain.ParallelNumber = 1;
            constrain.ThreadNumber = 10;
            constrain.SearchType = EnumerationType.ParallelSearchType.maximumClique;
            constrain.subGraphSize = 10;
            constrain.ipInitLb = false;
            constrain.L1 = 3000;
            constrain.L2 = 1000.0 * 60 * 60 * 3;
            constrain.CT_prime = 1000.0 * 60 * 60 * 3;
            constrain.hasStep1Solutions = false;
            _TestConstrains.add(constrain);
        }
    }

    public static DecimalFormat df = new DecimalFormat("0000");
    public static int Loops = 1;

    public String Size;
    public String ConstrainCsv;
    public int LoopIdx;
    private MethodName _MethodName;
    private double TimeOver;
    public ITestGenerator Model;

    BaseTester(int loopIdx, TestConstrain con) {
        this.LoopIdx = loopIdx;
        this.Size = con.Size;
        this.ConstrainCsv = con.ConstrainCsv;
        this.TimeOver = con.TimeOver;
        this._MethodName = con._MethodName;

        switch (con._MethodName) {
            case HMCAPIP:
                this.Model = new HMCAPIP(con.ParallelNumber, con.ThreadNumber, null, con.SearchType,
                        con.subGraphSize, con.ipInitLb, con.L1, con.L2, con.CT_prime, con.hasStep1Solutions);
                break;
            case RIPMCP:
                this.Model = new RIPMCP(con.ThreadNumber, con.L1, con.L2, con.CT_prime);
                break;
            case RndMCP:
                this.Model = new RndMCP(con.L1, con.L2, con.CT_prime);
                break;
            default:
                break;
        }
    }

    public void run() {
        this.SetInitializeSetting();

        this.Model.Generate();

        this.OutputResult();
    }

    private void SetInitializeSetting() {
        this.Model.SetExecuteTime(this.TimeOver);

        String filename = ".//ItemDB//" + df.format(Integer.parseInt(this.Size)) + "_" + this.df.format(this.LoopIdx) + ".csv";

        UtiltyFunction.CreateItemDataBase(filename, Integer.parseInt(this.Size));

        Constraint con = Reader.ConstraintRead(".//Constraint//" + this.ConstrainCsv);

        ArrayList<Item> itmeDb = Reader.itemDBRead(filename);

        con.setItemBank(itmeDb);

        this.Model.SetConstraint(con);

        this.SetLogger();
    }

    private void SetLogger() {
        PrintWriter log = null;
        try {
            log = new PrintWriter(".//log//" + df.format(Integer.parseInt(this.Size)) +
                    "_" + df.format(this.LoopIdx) + this._MethodName + this.ConstrainCsv + ".log");
        } catch (Exception e) {
            e.printStackTrace();
        }

        this.Model.SetLogger(log);
    }

    private void OutputResult() {
        this.Model.PrintResultSize();
        String path = df.format(Integer.parseInt(this.Size)) + "_" + this.df.format(this.LoopIdx) + "_" + this.ConstrainCsv;
        this.Model.OutputResult(path);
    }

    public static void main(String[] args) {
        int times = 1;
        for (TestConstrain testConstrain : TestConstrains._TestConstrains) {
            for (int idx = 0; idx < Loops; idx++) {

                BaseTester test = new BaseTester(idx, testConstrain);
                /*
                test.start();
                if (times % 1 == 0) {
                    try {
                        Thread.sleep((long)testConstrain.TimeOver);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                */
                test.run();
            }
            times++;
        }
    }
}
