package logic;

import basis.BaseTestHybridRBPGenerator;
import basis.EnumerationType;
import lib.MTRandom;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Random;

public class HMCAPIP extends BaseTestHybridRBPGenerator {

    public HMCAPIP(int parallelNumber, int threadNumber, ArrayList<BitSet> initClique,
                   EnumerationType.ParallelSearchType searchType, int subGraphSize,
                   boolean ipLbInit, int L1, double L2, double CT_prime,
                   boolean hasStep1Solutions) {
        super.method = EnumerationType.MethodName.HMCAPIP;
        this.ParallelNumber = parallelNumber;
        this.ThreadNumber = threadNumber;
        super.CurrentClique = initClique;
        this.SearchType = searchType;
        this.subGraphSize = subGraphSize;
        this.ipLbInit = ipLbInit;
        super._C1 = L1;
        super._C2 = L2;
        super._timeOver = CT_prime;
        super.AlreadyInitSearch = hasStep1Solutions;
    }

    private long start;
    private double logspan;
    private long befLogTime;

    public int ParallelNumber;
    public int ThreadNumber;
    public EnumerationType.ParallelSearchType SearchType;

    public int subGraphSize;
    public boolean ipLbInit;

    @Override
    public void Generate() {
        System.out.println(String.valueOf(this.subGraphSize));
        System.out.println(String.valueOf(this.ipLbInit));
        System.out.println(String.valueOf(this.ParallelNumber));
        System.out.println(super.constraint.UBofOverlappingItems.get(0).toString());

        super.subGraphSize = this.subGraphSize;
        super.ipLbInit = this.ipLbInit;
        super.ParallelNum = this.ParallelNumber;
        super.ReadTests("");

        if (!super.InitializeCheck()) {
            return;
        }

        start = System.currentTimeMillis();
        logspan = 1000.0 * 60 * 10;
        befLogTime = System.currentTimeMillis();

        super.ExportCsv(0, 0, 0, System.currentTimeMillis());

        super.InitialSearch();

        super.ExportCsv(0, 0, super.CurrentClique.size(), System.currentTimeMillis());

        int add = 0;
        do {
            this.add();
            if (super.result.size() < super.CurrentClique.size()) {
                super.result = (ArrayList<BitSet>) super.CurrentClique.clone();
            }
        } while (super.timeOver > System.currentTimeMillis() - start);

        if (super.Log != null) {
            super.Log.close();
        }

        if (super.result.size() < super.CurrentClique.size()) {
            super.result = (ArrayList<BitSet>) super.CurrentClique.clone();
        }

        super.ExportLstCsv(this.ParallelNumber);
    }

    private void add() {

        boolean slv = true;
        do {

            ArrayList<BitSet> tmpV = new ArrayList<>();
            do {
                ArrayList<BitSet> e;

                if (this.SearchType == EnumerationType.ParallelSearchType.maximumClique) {
                    MultiThreadSolver a = new MultiThreadSolver();
                    a.setConstraint(super.constraint);
                    a.st = this.start;
                    a.limit = this.timeOver;
                    a.threadNum = this.ThreadNumber;
                    a.subGraphSize = this.subGraphSize;
                    a.ipLbInit = this.ipLbInit;

                    boolean output;
                    if (System.currentTimeMillis() - befLogTime > logspan) {
                        output = true;
                        befLogTime = System.currentTimeMillis();
                    } else {
                        output = false;
                    }

                    if ((e = a.assemble(super.CurrentClique, output, this.ParallelNumber)) != null) {
                        tmpV.addAll(e);
                        super.LpTotalVertices += tmpV.size();
                        super.LpTotalSolutions += a.TotalSolutions;
                        super.ExportCsv(a.TotalSolutions, tmpV.size(), super.CurrentClique.size(), System.currentTimeMillis());
                    } else {
                        slv = false;
                        super.ExportCsv(0, 0, super.CurrentClique.size(), System.currentTimeMillis());
                    }
                    System.gc();
                }

            } while (tmpV.size() < 1 && slv);

            if (slv) {
                super.CurrentClique.addAll(tmpV);
                System.out.print("|");
                System.out.print("size:" + super.CurrentClique.size());
                if (super.timeOver < System.currentTimeMillis() - start) {
                    return;
                }
            } else {
                this.remove();
            }

        } while (true);
    }

    private void remove() {
        if (super.timeOver < System.currentTimeMillis() - start) {
            return;
        }

        int rmTests = 100;
        System.out.println("remove done, size" + super.CurrentClique.size()  + "addcount");
        System.out.println("remove done, size" + rmTests + "clique");

        for (int i = 0; i < rmTests; i++) {
            Random r = new MTRandom(555);
            if (super.CurrentClique.size() > 0) {
                int idx = r.nextInt(super.CurrentClique.size());
                super.CurrentClique.remove(idx);
            }
        }
    }
}
