package logic;

import ilog.concert.*;
import ilog.cplex.IloCplex;
import util.Constraint;
import util.Item;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Random;

public class LpModel {

    private int itemBankSize;
    private ArrayList<Item> ItemDB = new ArrayList<>();

    public void setConstraint(Constraint con) {
        this.Constraint = con;
        this.ItemDB = Constraint.getItemBank();
    }

    private Random RANDOM = new Random();
    private Constraint Constraint = null;

    public IloNumVar[] NumVars;
    public IloNumExpr[][] InfoNumExprs;

    public IloRange[] DuplicateConstraints;
    public IloRange[] TestInfomationConstraints;
    public IloRange TestSizeConstraints;

    double[] randoms;

    public boolean subGraphFlag;

    public IloCplex GetModeler(ArrayList<BitSet> clique, ArrayList<BitSet> subGraph, boolean ipLbInit) {
        try {
            IloCplex cplex = new IloCplex();

            this.itemBankSize = ItemDB.size();

            this.SetRandoms();

            this.CreateConstraints(clique, subGraph, cplex, ipLbInit);

            cplex.addMaximize(this.GetObjectiveFunction(cplex));

            return cplex;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private void SetRandoms() {
        this.randoms = new double[itemBankSize];

        for (int i =0; i < itemBankSize; i++) {
            this.randoms[i] = RANDOM.nextDouble();
        }
    }

    public IloLinearNumExpr GetObjectiveFunction(IloCplex cplex) {
        try {
            IloLinearNumExpr objective = cplex.linearNumExpr();

            objective.addTerms(this.randoms, this.NumVars);

            return objective;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public void CreateConstraints(ArrayList<BitSet> clique, ArrayList<BitSet> subGraph, IloCplex cplex, boolean ipLbInit) {
        try {

            int constSize = this.Constraint.theta.size();

            this.NumVars = new IloNumVar[this.itemBankSize];
            this.InfoNumExprs = new IloNumExpr[constSize][this.itemBankSize];

            for (int i = 0; i < itemBankSize; i++) {

                Item item = this.ItemDB.get(i);

                this.NumVars[i] = cplex.boolVar(item.id);

                for (int j = 0; j < constSize; j++) {
                    this.InfoNumExprs[j][i] = cplex.prod(item.getInfo(this.Constraint.theta.get(j)), this.NumVars[i]);
                }
            }

            if (clique.size() != 0) {
                this.DuplicateConstraints = new IloRange[clique.size()];

                for (BitSet test : clique) {
                    IloNumVar[] itemVars = new IloNumVar[this.Constraint.NumOfItems.get(0)];
                    int idx = 0;

                    for (int i = 0; i < test.length(); i++) {
                        if (test.get(i)) {
                            itemVars[idx] = this.NumVars[i];
                            idx++;
                        }
                    }
                    IloRange range = cplex.addLe(cplex.sum(itemVars), this.Constraint.UBofOverlappingItems.get(0));
                }
            }

            this.TestSizeConstraints = cplex.addEq(cplex.sum(NumVars), this.Constraint.NumOfItems.get(0));

            this.TestInfomationConstraints = new IloRange[constSize];
            for (int k = 0; k < constSize; k++) {
                TestInfomationConstraints[k] = cplex.addRange(this.Constraint.InformationLB.get(k), cplex.sum(this.InfoNumExprs[k]), this.Constraint.InformationUB.get(k));
            }

            if (ipLbInit) {
                if (subGraph != null && 0 < subGraph.size()) {
                    double maxWeight = 0.0;
                    double[] bestSol = new double[this.itemBankSize];

                    for (BitSet node : subGraph) {
                        double weight = 0.0;
                        double[] sol = new double[this.itemBankSize];
                        for (int i = 0; i < this.itemBankSize; i++) {
                            if (node.get(i)) {
                                sol[i] = 1.0;
                                weight += this.randoms[i];
                            } else {
                                sol[i] = 0.0;
                            }
                        }
                        if (maxWeight < weight) {
                            maxWeight = weight;
                            bestSol = sol.clone();
                        }
                    }

                    cplex.addMIPStart(this.NumVars, bestSol);
                }
            }

        } catch (IloException exc) {
            System.err.println("Concert exception caught: " + exc);
        }
    }
}
