package logic;

import ilog.concert.IloException;
import ilog.concert.IloNumVar;
import ilog.cp.IloCP;
import ilog.cplex.IloCplex;
import util.Constraint;
import util.Item;
import util.UtiltyFunction;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Random;
import java.util.concurrent.Callable;

public class LpSearch implements Callable<IPResultModel> {
    private String PATH = ".//";
    private String LPPATH = PATH + "lp//";
    private String LOG_PATH = PATH + "log//";

    private boolean debug = true;

    public int itemBankSize;
    private ArrayList<Item> ItemDB = new ArrayList<>();
    private ArrayList<BitSet> Clique;
    private ArrayList<BitSet> Result;

    public IloCplex Cplex;

    public void setConstraint(Constraint con, ArrayList<BitSet> clique) {
        this.Constraint = con;
        this.ItemDB = Constraint.getItemBank();
        this.Clique = clique;
    }

    private Random RANDOM = new Random();
    private Constraint Constraint = null;

    public IloNumVar[] NumVars;
    public int threadNum;

    long start;

    public IPResultModel call() {
        return this.Solve();
    }

    public IPResultModel Solve() {

        try {
            IPResultModel model = new IPResultModel();
            ArrayList<BitSet> result = new ArrayList<>();

            this.Cplex.setParam(IloCplex.Param.MIP.Display, 0);

            this.Cplex.setParam(IloCplex.Param.Threads, this.threadNum);

            this.Cplex.setParam(IloCplex.Param.TimeLimit, 60);

            long st = System.currentTimeMillis();
            if (this.Cplex.solve()) {
                model.ct = System.currentTimeMillis() - st;

                result = new ArrayList<>();

                result.add(UtiltyFunction.ConvertSolutionToBitSet(this.Cplex.getValues(this.NumVars), this.itemBankSize));
                model.results = result;

                model.nodes = this.Cplex.getNnodes64();
            }

            this.Cplex.end();

            return model;
        } catch (IloException exc) {
            System.err.println("Concert exception caught: " + exc);
            return null;
        }
    }
}
