package logic;

import basis.BaseTestHybridRBPGenerator;
import basis.EnumerationType;
import lib.MTRandom;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Random;

public class RIPMCP extends BaseTestHybridRBPGenerator {

    public RIPMCP(int threadNumber, int L1, double L2, double CT_prime) {
        super.method = EnumerationType.MethodName.RIPMCP;
        this.ThreadNumber = threadNumber;
        super._C1 = L1;
        super._C2 = L2;
        super._timeOver = CT_prime;
        this.SearchType = EnumerationType.ParallelSearchType.maximumClique;
    }

    private long start;
    private double logspan;
    private long befLogTime;

    public int ThreadNumber;
    public EnumerationType.ParallelSearchType SearchType;

    @Override
    public void Generate() {
        if (!super.InitializeCheck()) {
            return;
        }

        start = System.currentTimeMillis();
        logspan = 1000.0 * 60 * 10;
        befLogTime = System.currentTimeMillis();

        super.ExportCsv(0, 0, 0, System.currentTimeMillis());

        ArrayList<BitSet> tests = new ArrayList<>();
        this.CurrentClique = (ArrayList<BitSet>) tests.clone();
        this.result = (ArrayList<BitSet>) tests.clone();

        do {
            this.add();
            if (super.result.size() < super.CurrentClique.size()) {
                super.result = (ArrayList<BitSet>) super.CurrentClique.clone();
            }
        } while (super.timeOver > System.currentTimeMillis() - start);

        if (super.Log != null) {
            super.Log.close();
        }

        if (super.result.size() < super.CurrentClique.size()) {
            super.result = (ArrayList<BitSet>) super.CurrentClique.clone();
        }
    }

    private void add() {

        boolean slv = true;
        do {

            ArrayList<BitSet> tmpV = new ArrayList<>();
            do {
                ArrayList<BitSet> e;

                if (this.SearchType == EnumerationType.ParallelSearchType.maximumClique) {
                    Solver a = new Solver();
                    a.setConstraint(super.constraint);
                    a.st = this.start;
                    a.limit = this.timeOver;
                    a.threadNum = this.ThreadNumber;
                    a.subGraphSize = super._C1;

                    boolean output;
                    if (System.currentTimeMillis() - befLogTime > logspan) {
                        output = true;
                        befLogTime = System.currentTimeMillis();
                    } else {
                        output = false;
                    }

                    if ((e = a.assemble(super.CurrentClique)) != null) {
                        tmpV.addAll(e);
                        super.LpTotalVertices += tmpV.size();
                        super.LpTotalSolutions += a.TotalSolutions;
                        super.ExportCsv(a.TotalSolutions, tmpV.size(), super.CurrentClique.size(), System.currentTimeMillis());
                    } else {
                        slv = false;
                        super.ExportCsv(0, 0, super.CurrentClique.size(), System.currentTimeMillis());
                    }
                    System.gc();
                }

            } while (tmpV.size() < 1 && slv);

            if (slv) {
                super.CurrentClique.addAll(tmpV);
                System.out.print("|");
                System.out.print("size:" + super.CurrentClique.size());
                if (super.timeOver < System.currentTimeMillis() - start) {
                    return;
                }
            } else {
                this.remove();
            }

        } while (true);
    }

    private void remove() {
        if (super.timeOver < System.currentTimeMillis() - start) {
            return;
        }

        int rmTests = 100;
        System.out.println("remove done, size" + super.CurrentClique.size()  + "addcount");
        System.out.println("remove done, size" + rmTests + "clique");

        for (int i = 0; i < rmTests; i++) {
            Random r = new MTRandom(555);
            if (super.CurrentClique.size() > 0) {
                int idx = r.nextInt(super.CurrentClique.size());
                super.CurrentClique.remove(idx);
            }
        }
    }
}
