package logic;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.Iterator;

import util.graph.GraphStructure;
import basis.BaseTestGenerator;
import basis.EnumerationType;

public class RndMCP extends BaseTestGenerator {
    public boolean debug = true;
    private long startTime = 0;

    public RndMCP(int L1, double L2, double CT) {
        method = EnumerationType.MethodName.RndMCP;
        this.C1 = L1;
        this.C2 = L2;
        super.timeOver = CT;
    }

    public int C1;

    public double C2;

    public PrintWriter Log = null;

    private boolean FullEdge;

    @Override
    public void Generate() {
        ArrayList<BitSet> SubGraphV;
        BitSet[] SubGAdjMatrix;

        if (super.constraint == null) {
            return;
        } else {
            startTime = System.currentTimeMillis();

            ArrayList<BitSet> CurrentMax = new ArrayList<>();
            long iterate = 0;
            do {
                SubGraphV = GenerateNodeByRandom();
                SubGAdjMatrix = CreateSubGraph(SubGraphV);

                GC();
                if (SubGAdjMatrix.length == 0) {
                    break;
                }

                if (!this.FullEdge) {
                    GraphStructure g = new GraphStructure();
                    g.SetAdjMatrix(SubGAdjMatrix);
                    System.out.println("MCP_start" + SubGraphV.size());
                    MaxClique MC = new MaxClique();
                    String state = MC.Solve(g, C2);
                    System.out.println("MCP_end");

                    if (MC.Qmax.size() > CurrentMax.size()) {
                        CurrentMax = new ArrayList<>();
                        Iterator<Integer> it = MC.Qmax.iterator();
                        while (it.hasNext()) {
                            int index = it.next();
                            BitSet tmp = (BitSet) SubGraphV.get(index).clone();
                            CurrentMax.add(tmp);
                        }
                        System.out.println("Current maximum clique size:" + CurrentMax.size());
                    }
                } else {
                    if (SubGraphV.size() > CurrentMax.size()) {
                         CurrentMax = new ArrayList<>();
                         CurrentMax.addAll(SubGraphV);
                    }
                }

                if (Log != null) {
                    Log.print(iterate + ",");
                    Log.print(System.currentTimeMillis() - startTime + ",");
                    Log.print(CurrentMax.size());
                    Log.println();
                }

                GC();
                iterate++;
            } while ((System.currentTimeMillis() - startTime) < super.timeOver);


            if (Log != null) {
                Log.close();
            }

            super.result = CurrentMax;

            return;
        }
    }


    private ArrayList<BitSet> GenerateNodeByRandom() {
        final double per = 0.02;
        final int oneStepProgress = (int) Math.round(C1 * per);

        if (debug) {
            System.out.println("-----GenRandomSubGraph()-----");
        }
        if (Log != null) {
            Log.println("progress of GenSubGraph");
        }
        if (debug) {
            System.out.println("-----addSubGraph-----");
        }

        ArrayList<BitSet> SubGraphV = new ArrayList<>();

        do {
            BitSet s = RandBitSet.get(super.constraint);
            if (super.constraint.Check(s)) {
                SubGraphV.add(s);

                if (SubGraphV.size() % oneStepProgress == 0) {
                    System.out.print("|");
                }
            }
            long diff = System.currentTimeMillis() - startTime;
            if (diff > super.timeOver) {
                break;
            }
        } while (SubGraphV.size() < C1);

        System.out.println("done");

        return SubGraphV;
    }

    private BitSet[] CreateSubGraph(ArrayList<BitSet> SubGraphV) {
        final double per = 0.02;
        final int oneStepProgress = (int) Math.round(C1 * per);

        if (debug) {
            System.out.println("-----genStructure-----");
        }

        BitSet[] SubGAdjMatrix = new BitSet[SubGraphV.size()];
        boolean fullEdge = true;

        for (int i = 0; i < SubGraphV.size(); i++) {
            SubGAdjMatrix[i] = new BitSet();
            BitSet testi = (BitSet) SubGraphV.get(i).clone();
            for (int j = 0; j < SubGraphV.size(); j++) {
                if (j < i) {
                    boolean b = SubGAdjMatrix[j].get(i);
                    SubGAdjMatrix[i].set(j, b);
                } else if (j == i) {
                    SubGAdjMatrix[i].clear(j);
                } else if (j > i) {
                    BitSet testj = (BitSet) SubGraphV.get(j).clone();
                    testj.and(testi);

                    boolean hasEdge = super.constraint.overlapCheck(testj);
                    SubGAdjMatrix[i].set(j, hasEdge);
                    if (!hasEdge) {
                        fullEdge = false;
                    }
                }
            }
            if (i % oneStepProgress == 0) {
                System.out.print("|");
            }
        }
        System.out.println("done");

        this.FullEdge = fullEdge;

        return SubGAdjMatrix;
    }

    private void GC() {
        System.gc();
        System.runFinalization();
        System.gc();
    }
}
