package logic;

import util.Constraint;
import util.Item;
import util.graph.GraphStructure;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.Iterator;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

public class Solver {
    private String PATH = ".//";
    private String LOG_PATH = PATH + "log//";

    private boolean debug = true;

    private ArrayList<Item> ItemDB = new ArrayList<>();

    public void setConstraint(Constraint con) {
        this.Constraint = con;
        this.ItemDB = Constraint.getItemBank();
    }

    private Constraint Constraint = null;

    long start;

    public long st;
    public double limit;

    public int TotalSolutions = 0;
    private boolean FullEdge;

    public int threadNum;

    public int subGraphSize;

    public ArrayList<BitSet> assemble(ArrayList<BitSet> clique) {
        start = System.currentTimeMillis();
        if (this.Constraint == null) {
            return null;
        } else {

            ArrayList<BitSet> result = null;
            ArrayList<BitSet> solutions = null;

            LpModel baseModel = new LpModel();
            baseModel.setConstraint(this.Constraint);
            baseModel.subGraphFlag = true;

            boolean endFlg = false;

            ArrayList<Long> nodeTotal = new ArrayList<>();
            ArrayList<Long> calTime = new ArrayList<>();
            ArrayList<Long> oh = new ArrayList<>();
            long mcTime = 0;

            do {
                final ExecutorService executor = Executors.newCachedThreadPool();
                ArrayList<Future<IPResultModel>> list = new ArrayList<>();

                    LpSearch solver = new LpSearch();
                    solver.Cplex = baseModel.GetModeler(clique, solutions, false);
                    solver.NumVars = baseModel.NumVars;
                    solver.itemBankSize = this.ItemDB.size();
                    solver.threadNum = this.threadNum;


                    IPResultModel ret = solver.call();

                    if (ret.results != null && 0 < ret.results.size()) {
                        if (solutions == null) {
                            solutions = new ArrayList<>();
                        }
                        solutions.addAll(ret.results);

                        nodeTotal.add(ret.nodes);

                        oh.add(ret.oh);

                        calTime.add(ret.ct);
                    } else {
                        executor.shutdownNow();
                        endFlg = true;
                        break;
                    }

                if (this.limit < (System.currentTimeMillis() - this.st - 1000.0 * 60 * 3)) {
                    break;
                }
            } while (solutions != null && solutions.size() < this.subGraphSize);

            if (solutions != null && 0 < solutions.size()) {
                this.TotalSolutions += solutions.size();

                long t = System.currentTimeMillis();
                result = this.SearchMaxClique(solutions);
                mcTime = System.currentTimeMillis() - t;
                this.ExportCsv(nodeTotal, calTime, oh, mcTime, 1);
            }

            return result;
        }
    }

    private ArrayList<BitSet> SearchMaxClique(ArrayList<BitSet> graph) {
        BitSet[] subGraph = this.SubGraph(graph);
        if (this.FullEdge) {
            return graph;
        }

        GraphStructure g = new GraphStructure();
        g.SetAdjMatrix(subGraph);

        System.out.println("MCP_start");
        MaxClique MC = new MaxClique();
        String state = MC.Solve(g, 1000.0 * 60 * 3);
        System.out.println("MCP_Done");

        ArrayList<BitSet> maxClique = new ArrayList<>();

        if (0 < MC.Qmax.size()) {
            Iterator<Integer> it = MC.Qmax.iterator();

            while (it.hasNext()) {
                int index = it.next();
                BitSet useItem = (BitSet) graph.get(index).clone();
                maxClique.add(useItem);
            }
        } else {
            maxClique.add(graph.get(0));
        }
        System.out.println("maxCliqueSize" + maxClique.size());

        return maxClique;
    }

    private BitSet[] SubGraph(ArrayList<BitSet> SubGraphV) {
        System.out.println("-----CreateSubGraph-----");
        BitSet[] SubGAdjMatrix = new BitSet[SubGraphV.size()];

        boolean fullEdge = true;

        for (int i = 0; i < SubGraphV.size(); i++) {
            SubGAdjMatrix[i] = new BitSet();
            BitSet testi = (BitSet) SubGraphV.get(i).clone();
            for (int j = 0; j < SubGraphV.size(); j++) {
                if (j < i) {
                    boolean b = SubGAdjMatrix[j].get(i);
                    SubGAdjMatrix[i].set(j, b);
                } else if (j == i) {
                    SubGAdjMatrix[i].clear(j);
                } else if (j > i) {
                    BitSet testj = (BitSet) SubGraphV.get(j).clone();
                    testj.and(testi);

                    boolean hasEdge = this.Constraint.overlapCheck(testj);
                    SubGAdjMatrix[i].set(j, hasEdge);
                    if (!hasEdge) {
                        fullEdge = false;
                    }
                }
            }
        }
        System.out.println("-----DoneSubGraph");
        this.FullEdge = fullEdge;

        return SubGAdjMatrix;
    }

    public void ExportCsv(ArrayList<Long> _nodes, ArrayList<Long> _ct, ArrayList<Long> _oh, long _mcTime, int _parallelNum) {

        String fn = this.Constraint.getItemBank().size() + "_OC" + this.Constraint.UBofOverlappingItems +
                "_L2Size" + String.valueOf(this.subGraphSize);

        try {
            FileWriter f = new FileWriter(LOG_PATH + fn + "_sub.csv", true);
            PrintWriter p = new PrintWriter(new BufferedWriter(f));

            for (int i = 0; i < _nodes.size(); i++) {
                p.print(_nodes.get(i));
                p.print(",");
                p.print(_ct.get(i));
                p.print(",");
                p.print(_mcTime);
                p.print(",");
                p.print(_oh.get(i));
                p.print(",");
                p.print(System.currentTimeMillis());
                p.println();
            }

            p.close();

        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }
}