package util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.BitSet;
import java.util.Iterator;
import java.util.List;

public class Constraint {
    private boolean debug = false;

    public int TotalItems;

    public ArrayList<Double> theta;
    public ArrayList<Double> InformationUB;
    public ArrayList<Double> InformationLB;

    public ArrayList<String> Fields;
    public ArrayList<Integer> NumOfItems;
    public ArrayList<Integer> UBofOverlappingItems;

    public Double TimeUB;
    public Double TimeLB;

    protected ArrayList<Item> ItemBank = null;
    public ArrayList<BitSet> fieldItemMask = null;

    public boolean Check(BitSet s) {

        for (int f = 0; f < fieldItemMask.size(); f++) {
            BitSet tmp = (BitSet) fieldItemMask.get(f).clone();
            tmp.and(s);
            if (tmp.cardinality() != NumOfItems.get(f)) {
                if (debug) System.out.println("feild false," + f + "," + tmp.cardinality());
                return false;
            }
        }


        double[] info = new double[theta.size()];


        for (int i = 0; i < ItemBank.size(); i++) {
            if (s.get(i)) {
                for (int t = 0; t < info.length; t++) {
                    info[t] += ItemBank.get(i).getInfo(theta.get(t));
                }

            }
        }

        for (int t = 0; t < info.length; t++) {
            if (info[t] < InformationLB.get(t) || info[t] > InformationUB.get(t)) {
                if (debug) System.out.println("false," + Arrays.toString(info));
                return false;
            }
        }

        return true;
    }


    public boolean overlapCheck(BitSet overlap) {

        for (int i = 0; i < UBofOverlappingItems.size(); i++) {
            BitSet tmp = (BitSet) fieldItemMask.get(i).clone();
            tmp.and(overlap);
            if (tmp.cardinality() > UBofOverlappingItems.get(i)) {
                return false;
            }
        }

        return true;
    }


    public boolean isConnected(List<BitSet> cut, BitSet v) {
        Iterator<BitSet> thiss = cut.iterator();
        boolean canAdd = true;
        BitSet target = v;
        while (thiss.hasNext()) {
            BitSet chekker = (BitSet) thiss.next().clone();
            chekker.and(target);
            boolean j = overlapCheck(chekker);
            if (!j) {
                canAdd = false;
                break;
            }
        }
        return canAdd;


    }

    public ArrayList<BitSet> connectedTests(List<BitSet> cut, List<BitSet> from) {
        ArrayList<BitSet> r = new ArrayList<BitSet>();

        Iterator<BitSet> thiss = cut.iterator();
        Iterator<BitSet> fromm = from.iterator();


        while (fromm.hasNext()) {

            BitSet target = (BitSet) fromm.next();

            if (isConnected(cut, target)) {
                r.add(target);
            }

        }


        return r;

    }

    @Override
    public String toString() {

        String ls = System.getProperty("line.separator");
        String r = "";
        r = r + "time LB/UB," + this.TimeLB + "," + this.TimeUB + ls;
        r = r + "theta,infoLB,UB" + ls;
        for (int i = 0; i < this.theta.size(); i++) {
            r = r + this.theta.get(i) + "," + this.InformationLB.get(i) + "," + this.InformationUB.get(i) + ls;
        }
        r = r + "fieldID,#Item,UBofOverlap" + ls;
        for (int i = 0; i < this.Fields.size(); i++) {
            r = r + this.Fields.get(i) + "," + this.NumOfItems.get(i) + "," + this.UBofOverlappingItems.get(i) + ls;
        }

        return r;

    }

    public ArrayList<Item> getItemBank() {
        return ItemBank;
    }

    public void setItemBank(ArrayList<Item> itemBank) {
        ItemBank = itemBank;
        fieldItemMask = new ArrayList<BitSet>();

        for (int i = 0; i < Fields.size(); i++) {
            String tmpf = Fields.get(i);
            BitSet FMask = new BitSet();
            for (int j = 0; j < ItemBank.size(); j++) {
                if (tmpf.equals(ItemBank.get(j).field)) {
                    FMask.set(j);
                }
            }
            fieldItemMask.add(FMask);
        }
    }

    public void print() {
        System.out.println("TotalItems: " + TotalItems);
        for (int i = 0; i < theta.size(); i++) System.out.println("theta" + i + ": " + theta.get(i));
        for (int i = 0; i < InformationUB.size(); i++)
            System.out.println("InformationUB" + i + ": " + InformationUB.get(i));
        for (int i = 0; i < InformationLB.size(); i++)
            System.out.println("InformationLB" + i + ": " + InformationLB.get(i));

        for (int i = 0; i < Fields.size(); i++) System.out.println("Fields" + i + ": " + Fields.get(i));
        for (int i = 0; i < NumOfItems.size(); i++) System.out.println("NumOfItems" + i + ": " + NumOfItems.get(i));
        for (int i = 0; i < UBofOverlappingItems.size(); i++)
            System.out.println("UBofOverlappingItems" + i + ": " + UBofOverlappingItems.get(i));

        System.out.println("TimeUB: " + TimeUB);
        System.out.println("TimeLB: " + TimeLB);
    }


}